@@ -2,6 +2,9 @@
 A module implementing a channel environment.
 """
 
+# built-in
+from typing import Iterator as _Iterator
+
 # internal
 from runtimepy.channel.environment.array import (
     ArrayChannelEnvironment as _ArrayChannelEnvironment,
@@ -12,15 +15,16 @@ from runtimepy.channel.environment.create import (
 from runtimepy.channel.environment.file import (
     FileChannelEnvironment as _FileChannelEnvironment,
 )
-from runtimepy.channel.environment.names import (
-    ChannelNameEnvironment as _ChannelNameEnvironment,
-)
 
 
 class ChannelEnvironment(
     _ArrayChannelEnvironment,
     _FileChannelEnvironment,
     _CreateChannelEnvironment,
-    _ChannelNameEnvironment,
 ):
     """A class integrating channel and enumeration registries."""
+
+    @property
+    def names(self) -> _Iterator[str]:
+        """Iterate over registered names in the environment."""
+        yield from self.channels.names.names
