@@ -23,6 +23,8 @@ from runtimepy.channel import IntChannel as _IntChannel
 from runtimepy.channel.registry import ChannelRegistry as _ChannelRegistry
 from runtimepy.enum import RuntimeEnum as _RuntimeEnum
 from runtimepy.enum.registry import EnumRegistry as _EnumRegistry
+from runtimepy.mixins.finalize import FinalizeMixin
+from runtimepy.primitives import StrToBool
 from runtimepy.primitives.field import BitField as _BitField
 from runtimepy.primitives.field.fields import BitFields as _BitFields
 from runtimepy.primitives.field.manager import (
@@ -38,8 +40,10 @@ BitfieldResult = _Tuple[_BitField, _Optional[_RuntimeEnum]]
 BoolChannelResult = _Tuple[_BoolChannel, _Optional[_RuntimeEnum]]
 IntChannelResult = _Tuple[_IntChannel, _Optional[_RuntimeEnum]]
 
+FieldOrChannel = _Union[_BitField, _AnyChannel]
 
-class BaseChannelEnvironment(_NamespaceMixin):
+
+class BaseChannelEnvironment(_NamespaceMixin, FinalizeMixin):
     """A class integrating channel and enumeration registries."""
 
     def __init__(
@@ -53,7 +57,10 @@ class BaseChannelEnvironment(_NamespaceMixin):
     ) -> None:
         """Initialize this channel environment."""
 
-        super().__init__(namespace=namespace, namespace_delim=namespace_delim)
+        _NamespaceMixin.__init__(
+            self, namespace=namespace, namespace_delim=namespace_delim
+        )
+        FinalizeMixin.__init__(self)
 
         if channels is None:
             channels = _ChannelRegistry()
@@ -62,7 +69,9 @@ class BaseChannelEnvironment(_NamespaceMixin):
 
         self.channels = channels
         self.enums = enums
-        self.fields = _BitFieldsManager(channels.names, enums, fields=fields)
+        self.fields = _BitFieldsManager(
+            channels.names, self.enums, fields=fields
+        )
 
         # Keep a mapping of each channel's name and integer identifier to the
         # underlying enumeration.
@@ -111,13 +120,19 @@ class BaseChannelEnvironment(_NamespaceMixin):
                 is_int = chan.raw.kind.is_integer
 
                 if is_int or chan.raw.kind.is_float:
-                    kind = int if is_int else float
+                    kind = int if is_int and not chan.raw.scaling else float
                     try:
                         value = kind(value)
                         resolved = True
                     except ValueError:
                         pass
 
+                # Handle booleans.
+                else:
+                    parsed = StrToBool.parse(value)
+                    value = parsed.result
+                    resolved = parsed.valid
+
                 if not resolved:
                     raise ValueError(
                         (
@@ -134,7 +149,7 @@ class BaseChannelEnvironment(_NamespaceMixin):
                 )
 
         # Assign the value to the channel.
-        chan.raw.value = value  # type: ignore
+        chan.raw.scaled = value  # type: ignore
 
     def apply(self, values: ValueMap) -> None:
         """Apply a map of values to the environment."""
@@ -161,16 +176,34 @@ class BaseChannelEnvironment(_NamespaceMixin):
 
         chan, enum = self[key]
 
-        value: ChannelValue
+        value: ChannelValue = chan.raw.scaled
 
         # Resolve enumeration values to strings.
         if enum is not None and resolve_enum:
-            value = enum.get_str(_cast(int, chan.raw.value))
-        else:
-            value = chan.raw.value
+            value = enum.get_str(_cast(int, value))
 
         return value
 
+    def exists(self, val: _RegistryKey) -> bool:
+        """Determine if a channel exists."""
+        return self.fields.has_field(val) or self.get(val) is not None
+
+    def field_or_channel(self, val: _RegistryKey) -> _Optional[FieldOrChannel]:
+        """Attempt to look up a field or channel for a given registry key."""
+
+        channel: _Optional[FieldOrChannel] = None
+
+        chan = self.get(val)
+        if chan is None:
+            # Check if the name is a field.
+            field = self.fields.get_field(val)
+            if field is not None:
+                channel = field
+        else:
+            channel, _ = chan
+
+        return channel
+
     def get(self, val: _RegistryKey) -> _Optional[ChannelResult]:
         """Attempt to get a channel and its enumeration (if it has one)."""
 
@@ -227,3 +260,14 @@ class BaseChannelEnvironment(_NamespaceMixin):
         return bool(
             self.channels == other.channels and self.enums == other.enums
         )
+
+    def age_ns(self, key: _RegistryKey) -> int:
+        """Get the age of an entity based on registry key."""
+
+        chan = self.get(key)
+        if chan is not None:
+            prim = chan[0].raw
+        else:
+            prim = self.fields[key].raw
+
+        return prim.age_ns()
