@@ -1,7 +1,7 @@
 # =====================================
 # generator=datazen
-# version=3.1.3
-# hash=9e87f6454df00f68a60186c6e98c7f81
+# version=3.1.4
+# hash=89bfb0e2f56effabf3cb8f70d6349c13
 # =====================================
 
 """
@@ -17,6 +17,8 @@ from vcorelib.args import CommandRegister as _CommandRegister
 
 # internal
 from runtimepy.commands.arbiter import add_arbiter_cmd
+from runtimepy.commands.server import add_server_cmd
+from runtimepy.commands.task import add_task_cmd
 from runtimepy.commands.tui import add_tui_cmd
 
 
@@ -29,6 +31,16 @@ def commands() -> _List[_Tuple[str, str, _CommandRegister]]:
             "run a connection-arbiter application from a config",
             add_arbiter_cmd,
         ),
+        (
+            "server",
+            "run a server for a specific connection factory",
+            add_server_cmd,
+        ),
+        (
+            "task",
+            "run a task from a specific task factory",
+            add_task_cmd,
+        ),
         (
             "tui",
             "run a terminal interface for the channel environment",
