@@ -12,8 +12,7 @@ from vcorelib.args import CommandFunction as _CommandFunction
 from vcorelib.asyncio import run_handle_stop as _run_handle_stop
 
 # internal
-from runtimepy.commands.common import arbiter_args
-from runtimepy.commands.tui import curses_wrap_if
+from runtimepy.commands.common import arbiter_args, curses_wrap_if
 from runtimepy.net.arbiter import ConnectionArbiter
 from runtimepy.tui.channels import CursesWindow as _CursesWindow
 
@@ -24,7 +23,9 @@ async def entry(
     """The async command entry."""
 
     arbiter = ConnectionArbiter(stop_sig=stop_sig, window=window)
-    await arbiter.load_configs(args.configs)
+
+    await arbiter.load_configs(args.configs, wait_for_stop=args.wait_for_stop)
+
     return await arbiter.app()
 
 
@@ -52,5 +53,7 @@ def arbiter_cmd(args: _Namespace) -> int:
 def add_arbiter_cmd(parser: _ArgumentParser) -> _CommandFunction:
     """Add arbiter-command arguments to its parser."""
 
-    arbiter_args(parser)
+    with arbiter_args(parser):
+        pass
+
     return arbiter_cmd
