@@ -3,6 +3,7 @@ A module aggregating commonly used networking interface.
 """
 
 # internal
+from runtimepy.net.backoff import ExponentialBackoff
 from runtimepy.net.connection import (
     BinaryMessage,
     Connection,
@@ -19,6 +20,7 @@ from runtimepy.net.util import (
     hostname,
     normalize_host,
     sockname,
+    try_log_connection_error,
 )
 
 __all__ = [
@@ -35,4 +37,6 @@ __all__ = [
     "hostname",
     "normalize_host",
     "sockname",
+    "ExponentialBackoff",
+    "try_log_connection_error",
 ]
