@@ -23,13 +23,20 @@ from vcorelib.namespace import Namespace as _Namespace
 from vcorelib.namespace import NamespaceMixin as _NamespaceMixin
 
 # internal
+from runtimepy.channel.environment.command import (
+    clear_env,
+    env_json_data,
+    register_env,
+)
 from runtimepy.net.arbiter.housekeeping import metrics_poller
 from runtimepy.net.arbiter.info import AppInfo, ConnectionMap
+from runtimepy.net.arbiter.result import AppResult, ResultState
 from runtimepy.net.arbiter.task import (
     ArbiterTaskManager as _ArbiterTaskManager,
 )
 from runtimepy.net.connection import Connection as _Connection
 from runtimepy.net.manager import ConnectionManager as _ConnectionManager
+from runtimepy.net.server import RuntimepyServerConnection
 from runtimepy.tui.mixin import CursesWindow, TuiMixin
 
 NetworkApplication = _Callable[[AppInfo], _Awaitable[int]]
@@ -40,7 +47,7 @@ ServerTask = _Awaitable[None]
 async def init_only(app: AppInfo) -> int:
     """A network application that doesn't do anything."""
 
-    del app
+    await app.all_finalized()
     return 0
 
 
@@ -152,6 +159,17 @@ class BaseConnectionArbiter(_NamespaceMixin, _LoggerMixin, TuiMixin):
 
         return result
 
+    def _setup_server_json(self, info: AppInfo) -> None:
+        """Add runtime data to the server's JSON data structure."""
+
+        cls = RuntimepyServerConnection
+
+        # Connect configuration data.
+        cls.json_data["config"] = info.original_config()
+
+        # Connect environment data.
+        cls.json_data["environments"] = env_json_data()
+
     async def _entry(
         self,
         app: NetworkApplicationlike = None,
@@ -164,6 +182,7 @@ class BaseConnectionArbiter(_NamespaceMixin, _LoggerMixin, TuiMixin):
         """
 
         result = -1
+        info: Optional[AppInfo] = None
 
         try:
             # Wait for servers to start.
@@ -186,6 +205,13 @@ class BaseConnectionArbiter(_NamespaceMixin, _LoggerMixin, TuiMixin):
 
             tasks = {x.name: x for x in self.task_manager.tasks}
 
+            # Register environments.
+            clear_env()
+            for task in tasks.values():
+                register_env(task.name, task.command)
+            for name, conn in self._connections.items():
+                register_env(name, conn.command)
+
             # Run application, but only if all the registered connections are
             # still alive after initialization.
             if not check_connections or not any(
@@ -198,11 +224,14 @@ class BaseConnectionArbiter(_NamespaceMixin, _LoggerMixin, TuiMixin):
                         self.logger,
                         stack,
                         self._connections,
+                        self.manager,
                         self._namespace,
                         self.stop_sig,
                         config if config is not None else self._config,
                         self,
                         tasks,  # type: ignore
+                        self.task_manager,
+                        [],
                     )
 
                     # Initialize tasks.
@@ -210,8 +239,13 @@ class BaseConnectionArbiter(_NamespaceMixin, _LoggerMixin, TuiMixin):
                     await _asyncio.gather(
                         *(x.init(info) for x in self.task_manager.tasks)
                     )
+                    for task in self.task_manager.tasks:
+                        task.env.finalize(strict=False)
                     self.logger.debug("Periodic tasks initialized.")
 
+                    # Wire runtime data to server JSON.
+                    self._setup_server_json(info)
+
                     # Start tasks.
                     self.logger.debug("Starting periodic tasks...")
                     await stack.enter_async_context(
@@ -230,11 +264,21 @@ class BaseConnectionArbiter(_NamespaceMixin, _LoggerMixin, TuiMixin):
                         if result == 0:
                             result = await self._run_apps(curr_app, info)
 
+                        # Populate "not run" statuses.
+                        else:
+                            info.results.append(
+                                [AppResult(app.__name__) for app in curr_app]
+                            )
+
         finally:
             for conn in self._connections.values():
                 conn.disable(f"app exit {result}")
             self.stop_sig.set()
 
+            # Summarize results.
+            if info is not None:
+                info.result(logger=self.logger)
+
         return result
 
     async def _run_apps(
@@ -244,6 +288,9 @@ class BaseConnectionArbiter(_NamespaceMixin, _LoggerMixin, TuiMixin):
 
         pairs = [(app, info.with_new_logger(app.__name__)) for app in apps]
 
+        # Pre-populate stage results with "not run" placeholders.
+        stage_results = [AppResult(app.__name__) for app in apps]
+
         for _, inf in pairs:
             inf.logger.info("Starting.")
 
@@ -254,10 +301,22 @@ class BaseConnectionArbiter(_NamespaceMixin, _LoggerMixin, TuiMixin):
                 pairs[idx][1].logger.info("Returned %d.", result)
                 total += result
 
-        except AssertionError as exc:
-            info.logger.exception("Failed an assertion:", exc_info=exc)
+                # Capture a normal result.
+                stage_results[idx] = AppResult(
+                    apps[idx].__name__, ResultState.from_int(result), result
+                )
+
+        # Keep track of stages that raise an exception.
+        except Exception as exc:  # pylint: disable=broad-exception-caught
+            stage_results = [
+                AppResult(app.__name__, ResultState.EXCEPTION, exception=exc)
+                for app in apps
+            ]
             total = -1
 
+        # Keep track of this stage's results.
+        info.results.append(stage_results)
+
         return total
 
     async def app(
@@ -277,6 +336,8 @@ class BaseConnectionArbiter(_NamespaceMixin, _LoggerMixin, TuiMixin):
             self.manager.manage(self.stop_sig),
             *self._servers,
         )
+        assert result is not None
+        assert result[0] is not None
         return int(result[0])
 
     def run(
