@@ -16,11 +16,14 @@ from vcorelib.logging import LoggerType as _LoggerType
 
 # internal
 from runtimepy.channel.environment import ChannelEnvironment
-from runtimepy.channel.environment.command import ChannelCommandProcessor
+from runtimepy.channel.environment.command.processor import (
+    ChannelCommandProcessor,
+)
 from runtimepy.metrics import ConnectionMetrics
 from runtimepy.mixins.environment import ChannelEnvironmentMixin
 from runtimepy.mixins.logging import LoggerMixinLevelControl
-from runtimepy.primitives import Bool
+from runtimepy.net.backoff import ExponentialBackoff
+from runtimepy.primitives import Bool, Uint8
 from runtimepy.primitives.byte_order import DEFAULT_BYTE_ORDER, ByteOrder
 
 BinaryMessage = _Union[bytes, bytearray, memoryview]
@@ -31,9 +34,12 @@ class Connection(LoggerMixinLevelControl, ChannelEnvironmentMixin, _ABC):
 
     uses_text_tx_queue = True
     uses_binary_tx_queue = True
+    connected = True
 
     byte_order: ByteOrder = DEFAULT_BYTE_ORDER
 
+    default_auto_restart = False
+
     def __init__(
         self,
         logger: _LoggerType,
@@ -43,7 +49,6 @@ class Connection(LoggerMixinLevelControl, ChannelEnvironmentMixin, _ABC):
         """Initialize this connection."""
 
         LoggerMixinLevelControl.__init__(self, logger=logger)
-        self._enabled = Bool(True)
 
         # A queue for out-going text messages. Connections that don't use
         # this can set 'uses_text_tx_queue' to False to avoid scheduling a
@@ -59,7 +64,7 @@ class Connection(LoggerMixinLevelControl, ChannelEnvironmentMixin, _ABC):
 
         self._tasks: _List[_asyncio.Task[None]] = []
         self.initialized = _asyncio.Event()
-        self.disabled_event = _asyncio.Event()
+        self.exited = _asyncio.Event()
 
         self.metrics = ConnectionMetrics()
 
@@ -70,10 +75,47 @@ class Connection(LoggerMixinLevelControl, ChannelEnvironmentMixin, _ABC):
             self.register_connection_metrics(self.metrics)
 
         # State.
-        self.env.channel("enabled", self._enabled)
+        self._enabled = Bool()
+        self.disabled_event = _asyncio.Event()
+        self.env.channel(
+            "enabled",
+            self._enabled,
+            description="Whether or not this connection is enabled.",
+        )
+        self._set_enabled(True)
+
+        # Restart state and behavior.
+        self._restarts = Uint8()
+        self._restart_attempts = Uint8()
+        self.env.channel(
+            "restarts",
+            self._restarts,
+            description="Number of successful connection restarts.",
+        )
+        self.env.channel(
+            "restart_attempts",
+            self._restarts,
+            description="Number of connection restart attempts.",
+        )
+
+        self._auto_restart = Bool(self.default_auto_restart)
+        self.env.channel(
+            "auto_restart",
+            self._auto_restart,
+            commandable=True,
+            description=(
+                "Whether or not this connection will "
+                "automatically attempt re-connection."
+            ),
+        )
 
         self.init()
 
+    @property
+    def auto_restart(self) -> bool:
+        """Determine if this connection should be automatically restarted."""
+        return bool(self._auto_restart)
+
     def init(self) -> None:
         """Initialize this instance."""
 
@@ -131,13 +173,22 @@ class Connection(LoggerMixinLevelControl, ChannelEnvironmentMixin, _ABC):
     def disable_extra(self) -> None:
         """Additional tasks to perform when disabling."""
 
+    def _set_enabled(self, state: bool) -> None:
+        """Set the enabled state for this connection."""
+
+        self._enabled.value = state
+        if not state:
+            self.disabled_event.set()
+            self.initialized.clear()
+        else:
+            self.disabled_event.clear()
+
     def disable(self, reason: str) -> None:
         """Disable this connection."""
 
         if self._enabled:
             self.logger.info("Disabling connection: '%s'.", reason)
             self.disable_extra()
-            self._enabled.value = False
 
             # Cancel tasks.
             for task in self._tasks:
@@ -145,10 +196,11 @@ class Connection(LoggerMixinLevelControl, ChannelEnvironmentMixin, _ABC):
                     task.cancel()
 
             # Signal that this connection has been disabled.
-            self.disabled_event.set()
+            self._set_enabled(False)
 
     async def _wait_sig(self, stop_sig: _asyncio.Event) -> None:
         """Disable the connection if a stop signal gets set."""
+
         await stop_sig.wait()
         self.disable("stop signal")
 
@@ -159,18 +211,70 @@ class Connection(LoggerMixinLevelControl, ChannelEnvironmentMixin, _ABC):
             self.disable("init failed")
         else:
             self.logger.info("Initialized.")
+
+        self.env.finalize(strict=False)
         self.initialized.set()
 
-    async def process(self, stop_sig: _asyncio.Event = None) -> None:
+    async def restart(self) -> bool:
+        """
+        Reset necessary underlying state for this connection to 'process'
+        again.
+        """
+        raise NotImplementedError
+
+    async def disable_in(self, time: float) -> None:
+        """A method for disabling a connection after some delay."""
+
+        await _asyncio.sleep(time)
+        self.disable(f"timed disable ({time}s)")
+
+    async def _handle_restart(
+        self,
+        stop_sig: _asyncio.Event = None,
+        backoff: ExponentialBackoff = None,
+    ) -> None:
+        """Handle exponential backoff when restoring connections."""
+
+        if backoff is None:
+            backoff = ExponentialBackoff()
+
+        while (
+            self.disabled
+            and not backoff.give_up
+            and (stop_sig is None or not stop_sig.is_set())
+        ):
+            await backoff.sleep()
+            if await self.restart():
+                self._set_enabled(True)
+                self._restarts.raw.value += 1
+
+            self._restart_attempts.raw.value += 1
+
+    async def process(
+        self,
+        stop_sig: _asyncio.Event = None,
+        disable_time: float = None,
+        backoff: ExponentialBackoff = None,
+    ) -> None:
         """
         Process tasks for this connection while the connection is active.
         """
 
+        await self._handle_restart(stop_sig=stop_sig, backoff=backoff)
+        if self.disabled:
+            return
+
         self._tasks = [
             _asyncio.create_task(self._process_read()),
             _asyncio.create_task(self._async_init()),
         ]
 
+        # Disable the connection automatically if requested.
+        if disable_time is not None:
+            self._tasks.append(
+                _asyncio.create_task(self.disable_in(disable_time))
+            )
+
         if self.uses_text_tx_queue:
             self._tasks.append(
                 _asyncio.create_task(self._process_write_text())
@@ -184,12 +288,14 @@ class Connection(LoggerMixinLevelControl, ChannelEnvironmentMixin, _ABC):
         if stop_sig is not None:
             self._tasks.append(_asyncio.create_task(self._wait_sig(stop_sig)))
 
+        self.exited.clear()
         await _asyncio.wait(self._tasks, return_when=_asyncio.ALL_COMPLETED)
 
         # Ensure that tasks have their exceptions retrieved.
         _log_exceptions(self._tasks, self.logger)
 
         await self.close()
+        self.exited.set()
 
     async def _process_read(self) -> None:
         """Process incoming messages while this connection is active."""
