@@ -6,22 +6,31 @@ from __future__ import annotations
 
 # built-in
 import asyncio as _asyncio
+from contextlib import suppress as _suppress
+from typing import Iterator as _Iterator
 from typing import List as _List
 from typing import Optional as _Optional
+from typing import Type as _Type
+from typing import TypeVar as _TypeVar
 
 # third-party
 from vcorelib.asyncio import log_exceptions as _log_exceptions
+from vcorelib.logging import LoggerMixin
 from vcorelib.math import default_time_ns as _default_time_ns
 
 # internal
 from runtimepy.net.connection import Connection as _Connection
 
+T = _TypeVar("T", bound=_Connection)
 
-class ConnectionManager:
+
+class ConnectionManager(LoggerMixin):
     """A class for managing connection processing at runtime."""
 
     def __init__(self) -> None:
         """Initialize this connection manager."""
+
+        super().__init__()
         self.queue: _asyncio.Queue[_Connection] = _asyncio.Queue()
         self._running = False
         self._conns: _List[_Connection] = []
@@ -31,6 +40,12 @@ class ConnectionManager:
         """Return the number of managed connections."""
         return len(self._conns)
 
+    def by_type(self, kind: _Type[T]) -> _Iterator[T]:
+        """Iterate over connections of a specific type."""
+        for conn in self._conns:
+            if isinstance(conn, kind):
+                yield conn
+
     def reset_metrics(self) -> None:
         """Reset connection metrics."""
         for conn in self._conns:
@@ -76,7 +91,19 @@ class ConnectionManager:
             next_tasks = _log_exceptions(tasks)
 
             # Filter out disabled connections.
-            self._conns = [x for x in self._conns if not x.disabled]
+            enabled = []
+            for conn in self._conns:
+                if not conn.disabled:
+                    enabled.append(conn)
+
+                # Check if this connection should be restarted.
+                elif conn.auto_restart:
+                    next_tasks.append(
+                        _asyncio.create_task(conn.process(stop_sig=stop_sig))
+                    )
+                    enabled.append(conn)
+
+            self._conns = enabled
 
             # If a new connection was made, register a task for processing
             # it.
@@ -90,10 +117,12 @@ class ConnectionManager:
 
             tasks = next_tasks
 
-        # Allow existing tasks to clean up.
-        if new_conn_task is not None:
-            new_conn_task.cancel()
-        for task in tasks:
-            await task
+        with self.log_time("Shutting down", reminder=True):
+            # Allow existing tasks to clean up.
+            if new_conn_task is not None:
+                new_conn_task.cancel()
+            if tasks:
+                with _suppress(_asyncio.TimeoutError):
+                    await _asyncio.wait_for(_asyncio.gather(*tasks), 1.0)
 
         self._running = False
