@@ -4,8 +4,6 @@ A module implementing a TCP connection interface.
 
 # built-in
 import asyncio as _asyncio
-from asyncio import BaseTransport as _BaseTransport
-from asyncio import Protocol as _Protocol
 from asyncio import Semaphore as _Semaphore
 from asyncio import Transport as _Transport
 from asyncio import get_event_loop as _get_event_loop
@@ -21,9 +19,6 @@ from typing import Type as _Type
 from typing import TypeVar as _TypeVar
 from typing import Union as _Union
 
-# third-party
-from vcorelib.logging import LoggerType as _LoggerType
-
 # internal
 from runtimepy.net import sockname as _sockname
 from runtimepy.net.connection import BinaryMessage as _BinaryMessage
@@ -31,42 +26,15 @@ from runtimepy.net.connection import Connection as _Connection
 from runtimepy.net.connection import EchoConnection as _EchoConnection
 from runtimepy.net.connection import NullConnection as _NullConnection
 from runtimepy.net.manager import ConnectionManager as _ConnectionManager
-from runtimepy.net.mixin import (
-    BinaryMessageQueueMixin as _BinaryMessageQueueMixin,
-)
 from runtimepy.net.mixin import TransportMixin as _TransportMixin
+from runtimepy.net.tcp.create import (
+    TcpTransportProtocol,
+    tcp_transport_protocol_backoff,
+    try_tcp_transport_protocol,
+)
+from runtimepy.net.tcp.protocol import QueueProtocol
 
 LOG = _getLogger(__name__)
-
-
-class QueueProtocol(_BinaryMessageQueueMixin, _Protocol):
-    """A simple streaming protocol that populates a message queue."""
-
-    logger: _LoggerType
-    conn: _Connection
-
-    def data_received(self, data: _BinaryMessage) -> None:
-        """Handle incoming data."""
-
-        self.queue.put_nowait(data)
-        self.queue_hwm = max(self.queue_hwm, self.queue.qsize())
-
-    def connection_made(self, transport: _BaseTransport) -> None:
-        """Log the connection establishment."""
-
-        self.logger = _getLogger(
-            _TransportMixin(transport).logger_name("TCP ")
-        )
-        self.logger.info("Connected.")
-
-    def connection_lost(self, exc: _Optional[Exception]) -> None:
-        """Log the disconnection."""
-
-        msg = "Disconnected." if exc is None else f"Disconnected: '{exc}'."
-        self.logger.info(msg)
-        self.conn.disable("disconnected")
-
-
 T = _TypeVar("T", bound="TcpConnection")
 ConnectionCallback = _Callable[[T], None]
 
@@ -85,10 +53,18 @@ class TcpConnection(_Connection, _TransportMixin):
 
         # Re-assign with updated type information.
         self._transport: _Transport = transport
+        self._set_protocol(protocol)
+
+        super().__init__(_getLogger(self.logger_name("TCP ")))
+
+        # Store connection-instantiation arguments.
+        self._conn_kwargs: dict[str, _Any] = {}
+
+    def _set_protocol(self, protocol: QueueProtocol) -> None:
+        """Set a new protocol for this instance."""
 
         self._protocol = protocol
         self._protocol.conn = self
-        super().__init__(_getLogger(self.logger_name("TCP ")))
 
     async def _await_message(self) -> _Optional[_Union[_BinaryMessage, str]]:
         """Await the next message. Return None on error or failure."""
@@ -107,17 +83,35 @@ class TcpConnection(_Connection, _TransportMixin):
         self._transport.write(data)
         self.metrics.tx.increment(len(data))
 
+    async def restart(self) -> bool:
+        """
+        Reset necessary underlying state for this connection to 'process'
+        again.
+        """
+
+        def callback(transport_protocol: TcpTransportProtocol) -> None:
+            """Callback if the socket creation succeeds."""
+
+            self.set_transport(transport_protocol[0])
+            self._set_protocol(transport_protocol[1])
+
+        result = await try_tcp_transport_protocol(
+            callback=callback, **self._conn_kwargs
+        )
+        return result is not None
+
     @classmethod
     async def create_connection(cls: _Type[T], **kwargs) -> T:
         """Create a TCP connection."""
 
-        eloop = _get_event_loop()
+        transport, protocol = await tcp_transport_protocol_backoff(**kwargs)
+        inst = cls(transport, protocol)
 
-        transport: _Transport
-        transport, protocol = await eloop.create_connection(
-            QueueProtocol, **kwargs
-        )
-        return cls(transport, protocol)
+        # Is there a better way to do this? We can't restart a server's side
+        # of a connection (seems okay).
+        inst._conn_kwargs = {**kwargs}
+
+        return inst
 
     @classmethod
     @_asynccontextmanager
@@ -182,7 +176,8 @@ class TcpConnection(_Connection, _TransportMixin):
         LOG.info("TCP Server closed.")
 
     @classmethod
-    async def create_pair(cls: _Type[T]) -> _Tuple[T, T]:
+    @_asynccontextmanager
+    async def create_pair(cls: _Type[T]) -> _AsyncIterator[_Tuple[T, T]]:
         """Create a connection pair."""
 
         cond = _Semaphore(0)
@@ -199,8 +194,8 @@ class TcpConnection(_Connection, _TransportMixin):
             conn2 = await cls.create_connection(host="localhost", port=host[1])
             await cond.acquire()
 
-        assert conn1 is not None
-        return conn1, conn2
+            assert conn1 is not None
+            yield conn1, conn2
 
     async def close(self) -> None:
         """Close this connection."""
