@@ -2,24 +2,18 @@
 A module implementing a UDP connection interface.
 """
 
-from __future__ import annotations
-
 # built-in
 from abc import abstractmethod as _abstractmethod
-import asyncio as _asyncio
-from asyncio import DatagramProtocol as _DatagramProtocol
 from asyncio import DatagramTransport as _DatagramTransport
 from contextlib import suppress as _suppress
 from logging import getLogger as _getLogger
 import socket as _socket
+from typing import Any as _Any
 from typing import Tuple as _Tuple
 from typing import Type as _Type
 from typing import TypeVar as _TypeVar
 from typing import Union as _Union
 
-# third-party
-from vcorelib.logging import LoggerType as _LoggerType
-
 # internal
 from runtimepy.net import IpHost, get_free_socket, normalize_host
 from runtimepy.net.connection import BinaryMessage as _BinaryMessage
@@ -27,34 +21,14 @@ from runtimepy.net.connection import Connection as _Connection
 from runtimepy.net.connection import EchoConnection as _EchoConnection
 from runtimepy.net.connection import NullConnection as _NullConnection
 from runtimepy.net.mixin import TransportMixin as _TransportMixin
+from runtimepy.net.udp.create import (
+    UdpTransportProtocol,
+    try_udp_transport_protocol,
+    udp_transport_protocol_backoff,
+)
+from runtimepy.net.udp.protocol import UdpQueueProtocol
 
 LOG = _getLogger(__name__)
-
-
-class UdpQueueProtocol(_DatagramProtocol):
-    """A simple UDP protocol that populates a message queue."""
-
-    logger: _LoggerType
-
-    def __init__(self) -> None:
-        """Initialize this protocol."""
-
-        self.queue: _asyncio.Queue[
-            _Tuple[_BinaryMessage, _Tuple[str, int]]
-        ] = _asyncio.Queue()
-        self.queue_hwm: int = 0
-
-    def datagram_received(self, data: bytes, addr: _Tuple[str, int]) -> None:
-        """Handle incoming data."""
-
-        self.queue.put_nowait((data, addr))
-        self.queue_hwm = max(self.queue_hwm, self.queue.qsize())
-
-    def error_received(self, exc: Exception) -> None:
-        """Log any received errors."""
-        self.logger.error(exc)
-
-
 T = _TypeVar("T", bound="UdpConnection")
 
 
@@ -71,13 +45,24 @@ class UdpConnection(_Connection, _TransportMixin):
         """Initialize this UDP connection."""
 
         _TransportMixin.__init__(self, transport)
+        if not self.remote_address:
+            self.connected = False
 
         # Re-assign with updated type information.
         self._transport: _DatagramTransport = transport
 
-        self._protocol = protocol
         super().__init__(_getLogger(self.logger_name("UDP ")))
+        self._set_protocol(protocol)
+
+        # Store connection-instantiation arguments.
+        self._conn_kwargs: dict[str, _Any] = {}
+
+    def _set_protocol(self, protocol: UdpQueueProtocol) -> None:
+        """Set a protocol instance for this connection."""
+
+        self._protocol = protocol
         self._protocol.logger = self.logger
+        self._protocol.conn = self
 
     def set_remote_address(self, addr: IpHost) -> None:
         """
@@ -119,14 +104,29 @@ class UdpConnection(_Connection, _TransportMixin):
         """Enqueue a binary message tos end."""
         self.sendto(data, addr=self.remote_address)
 
+    async def restart(self) -> bool:
+        """
+        Reset necessary underlying state for this connection to 'process'
+        again.
+        """
+
+        def callback(transport_protocol: UdpTransportProtocol) -> None:
+            """Callback if the socket creation succeeds."""
+
+            self.set_transport(transport_protocol[0])
+            self._set_protocol(transport_protocol[1])
+
+        result = await try_udp_transport_protocol(
+            callback=callback, **self._conn_kwargs
+        )
+        return result is not None
+
     @classmethod
     async def create_connection(
         cls: _Type[T], connect: bool = True, **kwargs
     ) -> T:
         """Create a UDP connection."""
 
-        eloop = _asyncio.get_event_loop()
-
         LOG.debug("kwargs: %s", kwargs)
 
         # If the caller specifies a remote address but doesn't want a connected
@@ -143,13 +143,10 @@ class UdpConnection(_Connection, _TransportMixin):
                 kwargs["local_addr"] = ("0.0.0.0", 0)
                 kwargs.setdefault("family", _socket.AF_INET)
 
-        transport: _DatagramTransport
-        (
-            transport,
-            protocol,
-        ) = await eloop.create_datagram_endpoint(UdpQueueProtocol, **kwargs)
-
+        # Create the underlying connection.
+        transport, protocol = await udp_transport_protocol_backoff(**kwargs)
         conn = cls(transport, protocol)
+        conn._conn_kwargs = {**kwargs}
 
         # Set the remote address manually if necessary.
         if not connect and remote_addr is not None:
