@@ -4,6 +4,7 @@ A module implementing a floating-point primitive interface.
 
 # internal
 from runtimepy.primitives.base import Primitive as _Primitive
+from runtimepy.primitives.scaling import ChannelScaling
 from runtimepy.primitives.type.float import Double as _Double
 from runtimepy.primitives.type.float import Float as _Float
 from runtimepy.primitives.type.float import Half as _Half
@@ -14,9 +15,11 @@ class HalfPrimitive(_Primitive[float]):
 
     kind = _Half
 
-    def __init__(self, value: float = 0.0) -> None:
+    def __init__(
+        self, value: float = 0.0, scaling: ChannelScaling = None
+    ) -> None:
         """Initialize this floating-point primitive."""
-        super().__init__(value=value)
+        super().__init__(value=value, scaling=scaling)
 
 
 Half = HalfPrimitive
@@ -27,9 +30,11 @@ class FloatPrimitive(_Primitive[float]):
 
     kind = _Float
 
-    def __init__(self, value: float = 0.0) -> None:
+    def __init__(
+        self, value: float = 0.0, scaling: ChannelScaling = None
+    ) -> None:
         """Initialize this floating-point primitive."""
-        super().__init__(value=value)
+        super().__init__(value=value, scaling=scaling)
 
 
 Float = FloatPrimitive
@@ -40,9 +45,11 @@ class DoublePrimitive(_Primitive[float]):
 
     kind = _Double
 
-    def __init__(self, value: float = 0.0) -> None:
+    def __init__(
+        self, value: float = 0.0, scaling: ChannelScaling = None
+    ) -> None:
         """Initialize this floating-point primitive."""
-        super().__init__(value=value)
+        super().__init__(value=value, scaling=scaling)
 
 
 Double = DoublePrimitive
