@@ -7,7 +7,9 @@ integer identifier.
 from abc import abstractmethod as _abstractmethod
 from typing import Dict as _Dict
 from typing import Generic as _Generic
+from typing import Iterator
 from typing import Optional as _Optional
+from typing import Tuple
 from typing import Type as _Type
 from typing import TypeVar as _TypeVar
 from typing import cast as _cast
@@ -49,6 +51,14 @@ class Registry(_RuntimepyDictCodec, _Generic[T]):
             reverse={name: item.id for name, item in self.items.items()}
         )
 
+    def search(
+        self, pattern: str, exact: bool = False
+    ) -> Iterator[Tuple[str, T]]:
+        """Search for items in the registry by name."""
+
+        for name in self.names.search(pattern, exact=exact):
+            yield name, self.items[name]
+
     def asdict(self) -> _JsonObject:
         """Get this registry as a dictionary."""
 
