@@ -2,13 +2,11 @@
 A module implementing an asynchronous task interface.
 """
 
+from runtimepy.metrics import PeriodicTaskMetrics
+
 # internal
 from runtimepy.task.asynchronous import AsyncTask
-from runtimepy.task.basic import (
-    PeriodicTask,
-    PeriodicTaskManager,
-    PeriodicTaskMetrics,
-)
+from runtimepy.task.basic import PeriodicTask, PeriodicTaskManager
 
 __all__ = [
     "AsyncTask",
