@@ -20,7 +20,9 @@ from vcorelib.math import rate_str as _rate_str
 
 # internal
 from runtimepy.channel.environment import ChannelEnvironment
-from runtimepy.channel.environment.command import ChannelCommandProcessor
+from runtimepy.channel.environment.command.processor import (
+    ChannelCommandProcessor,
+)
 from runtimepy.metrics import PeriodicTaskMetrics
 from runtimepy.mixins.environment import ChannelEnvironmentMixin
 from runtimepy.mixins.logging import LoggerMixinLevelControl
@@ -32,6 +34,8 @@ from runtimepy.primitives import Float as _Float
 class PeriodicTask(LoggerMixinLevelControl, ChannelEnvironmentMixin, _ABC):
     """A class implementing a simple periodic-task interface."""
 
+    auto_finalize = True
+
     def __init__(
         self,
         name: str,
@@ -63,9 +67,21 @@ class PeriodicTask(LoggerMixinLevelControl, ChannelEnvironmentMixin, _ABC):
         self.register_task_metrics(self.metrics)
 
         # State.
-        self.env.channel("paused", self._paused, commandable=True)
-        self.env.channel("period_s", self.period_s, commandable=True)
+        self.env.channel(
+            "paused",
+            self._paused,
+            commandable=True,
+            description="Whether or not this task is paused.",
+        )
+        self.env.channel(
+            "period_s",
+            self.period_s,
+            commandable=True,
+            description="Iteration period for this task.",
+        )
         self._init_state()
+        if self.auto_finalize:
+            self.env.finalize()
 
         self._dispatch_rate = _RateTracker(depth=average_depth)
         self._dispatch_time = _MovingAverage(depth=average_depth)
